﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using RoadTrain.DrawingObjects;
using RoadTrain.MovementStrategy;

namespace RoadTrain.Generics
{
    internal class TrainsGenericCollection<T, U>
        where T : DrawingRoadTrain
        where U : IMoveableObject
    {
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly int _placeSizeWidth = 220;
        private readonly int _placeSizeHeight = 100;
        private readonly SetGeneric<T> _collection;
        public TrainsGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }
        public static int operator +(TrainsGenericCollection<T, U> collect, T?
        obj)
        {
            if (obj == null)
            {
                return -1;
            }
            return collect._collection.Insert(obj);
        }
        public static bool operator -(TrainsGenericCollection<T, U> collect, int
        pos)
        {
            T? obj = collect._collection.Get(pos);
            if (obj != null)
            {
                return collect._collection.Remove(pos);
            }
            return false;
        }
        public U? GetU(int pos)
        {
            return (U?)_collection.Get(pos)?.GetMoveableObject;
        }
        public Bitmap ShowTrains()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }
        private void DrawObjects(Graphics g)
        {
            for (int i = 0; i < _collection.Count; i++)
            {
                DrawingRoadTrain train = _collection.Get(i);

                if (train != null)
                {
                    int width = _pictureWidth / _placeSizeWidth;
                    train.SetPosition(i % width * _placeSizeWidth, (i / (_pictureWidth / _placeSizeWidth)) * _placeSizeHeight);
                    train.DrawTransport(g);
                }
            }
        }
    }
}